@@ -19,12 +19,6 @@ $Yote::ObjProvider::PKG_TO_METHODS = {};
 $Yote::ObjProvider::WEAK_REFS      = {};
 $Yote::ObjProvider::LAST_LOAD_TIME = {};
 
-use constant LOCK_ON_WRITE => 1;
-use constant LOCK_ALWAYS   => 2;
-use constant LOCK_NEVER    => 3;
-
-our $LOCK_MODE = LOCK_ON_WRITE;
-
 our $DATASTORE;
 our $LOCKER;
 our $FIRST_ID;
@@ -41,7 +35,7 @@ sub new {
     my $ref = shift;
     my $class = ref( $ref ) || $ref;
     return bless {}, $class;
-}
+} #new
 
 sub init {
     my $args = ref( $_[0] ) ? $_[0] : { @_ };
@@ -57,7 +51,7 @@ sub init {
     $DATASTORE->ensure_datastore();
 } #init
 
-sub make_server {
+sub attach_server {
     $LOCKER = shift;
 }
 
@@ -69,15 +63,17 @@ sub commit_transaction {
     return $DATASTORE->commit_transaction();
 }
 
+sub count {
+    my( $container_id, $args ) = @_;
+    return $DATASTORE->count( $container_id, $args );
+}
+
 #
 # Markes given object as dirty.
 #
 sub dirty {
     my $obj = shift;
     my $id = shift;
-    if( $LOCK_MODE != LOCK_NEVER && $LOCKER ) {
-	$obj = $LOCKER->lock_object( $id, $obj );
-    }
     Yote::ObjManager::mark_dirty( $id );
     $Yote::ObjProvider::DIRTY->{$id} = $obj;
 } #dirty
@@ -88,13 +84,62 @@ sub disconnect {
 }
 
 #
-# Encrypt the password so its not saved in plain text.
+# Encrypt the password so its not saved in plain text. This is here because 
+# it fits best in the Object Provider, though it's not a great fit.
 #
 sub encrypt_pass {
     my( $pw, $handle ) = @_;
     return $handle ? Crypt::Passwd::XS::crypt( $pw, $handle ) : undef;
 } #encrypt_pass
 
+sub fetch {
+    my( $id ) = @_;
+
+    return undef unless $id;
+    #
+    # Return the object if we have a reference to its dirty state.
+    #
+    my $ref = $Yote::ObjProvider::DIRTY->{$id} || $Yote::ObjProvider::WEAK_REFS->{$id};
+
+    if( $ref ) {
+	return $ref;
+    }
+    my $obj_arry = $DATASTORE->fetch( $id );
+    $Yote::ObjProvider::LAST_LOAD_TIME->{$id} = time();
+
+    if( $obj_arry ) {
+        my( $id, $class, $data ) = @$obj_arry;
+	if( $class eq 'ARRAY' ) {
+	    my( @arry );
+	    tie @arry, 'Yote::Array', $id, @$data;
+	    my $tied = tied @arry; $tied->[2] = \@arry;
+	    __store_weak( $id, \@arry );
+	    return \@arry;
+	}
+	elsif( $class eq 'HASH' ) {
+	    my( %hash );
+	    tie %hash, 'Yote::Hash', $id, map { $_ => $data->{$_} } keys %$data;
+	    my $tied = tied %hash; $tied->[2] = \%hash;
+	    __store_weak( $id, \%hash );
+	    return \%hash;
+	}
+	else {
+	    eval("require $class");
+	    print STDERR Data::Dumper->Dump([$class,$!,$@]) if $@;
+	    die $@ if $@;
+
+	    my $obj = $class->new( $id );
+	    $obj->{DATA} = $data;
+	    $obj->{ID} = $id;
+	    $obj->_load();
+	    __store_weak( $id, $obj );
+	    return $obj;
+	}
+    }
+
+    return undef;
+} #fetch
+
 
 #
 # Returns the first ID that is associated with the root YoteRoot object
@@ -116,60 +161,6 @@ sub flush_all_volatile {
     $Yote::ObjProvider::WEAK_REFS = {};
 }
 
-sub fetch {
-    my( $id ) = @_;
-
-    return undef unless $id;
-    #
-    # Return the object if we have a reference to its dirty state.
-    #
-    my $ref = $Yote::ObjProvider::DIRTY->{$id} || $Yote::ObjProvider::WEAK_REFS->{$id};
-#	print STDERR "[$$ ".time()."] cached $ref $id, checking on LOCKER\n";
-    if( $LOCKER && $LOCK_MODE == LOCK_ALWAYS ) { 
-	$ref = $LOCKER->lock_object( $id, $ref );
-    }
-
-    if( $ref ) {
-#	print STDERR "[$$ ".time()."] returning ref $ref for $id\n";
-	return $ref;
-    }
-#    print STDERR "[$$ ".time()."] LOADING $id\n";
-    my $obj_arry = $DATASTORE->fetch( $id );
-    $Yote::ObjProvider::LAST_LOAD_TIME->{$id} = time();
-
-#    print STDERR "[$$ ".time()."] OBJ $id loaded at $Yote::ObjProvider::LAST_LOAD_TIME->{$id}\n";
-
-    if( $obj_arry ) {
-        my( $id, $class, $data ) = @$obj_arry;
-	if( $class eq 'ARRAY' ) {
-	    my( @arry );
-	    tie @arry, 'Yote::Array', $id, @$data;
-	    my $tied = tied @arry; $tied->[2] = \@arry;
-	    __store_weak( $id, \@arry );
-	    return \@arry;
-	}
-	elsif( $class eq 'HASH' ) {
-	    my( %hash );
-	    tie %hash, 'Yote::Hash', $id, map { $_ => $data->{$_} } keys %$data;
-	    my $tied = tied %hash; $tied->[2] = \%hash;
-	    __store_weak( $id, \%hash );
-	    return \%hash;
-	}
-	else {
-	    eval("require $class");
-	    print STDERR Data::Dumper->Dump([$class,$!,$@]) if $@;
-	    my $obj = $class->new( $id );
-	    $obj->{DATA} = $data;
-	    $obj->{ID} = $id;
-	    $obj->_load();
-	    __store_weak( $id, $obj );
-	    return $obj;
-	}
-    }
-
-    return undef;
-} #fetch
-
 
 sub get_id {
     my $ref = shift;
@@ -228,41 +219,51 @@ sub get_id {
 
 } #get_id
 
-sub list_insert {
-    my( $list_id, $val, $idx ) = @_;
-    return $DATASTORE->list_insert( $list_id, ref( $val ) ? get_id( $val ) : "v$val", $idx );
-}
-
-sub list_delete {
-    my( $list_id, $key ) = @_;
-    return $DATASTORE->list_delete( $list_id, $key );
-}
-
 sub hash_delete {
     my( $hash_id, $key ) = @_;
     return $DATASTORE->hash_delete( $hash_id, $key );
 }
 
-sub hash_insert {
-    my( $hash_id, $key, $val ) = @_;
-    return $DATASTORE->hash_insert( $hash_id, $key, ref( $val ) ? get_id( $val ) : "v$val" );
-} #hash_insert
-
 sub hash_fetch {
     my( $hash_id, $key ) = @_;
     return xform_out( $DATASTORE->hash_fetch( $hash_id, $key ) );
 } 
 
+sub hash_has_key {
+    my( $hash_id, $key ) = @_;
+    return $DATASTORE->hash_has_key( $hash_id, $key );
+}
+
+sub hash_insert {
+    my( $hash_id, $key, $val ) = @_;
+    return $DATASTORE->hash_insert( $hash_id, $key, xform_in( $val ) );
+} #hash_insert
+
+sub list_delete {
+    my( $list_id, $idx ) = @_;
+    return $DATASTORE->list_delete( $list_id, undef, $idx );
+}
+
 sub list_fetch {
     my( $list_id, $idx ) = @_;
     return xform_out( $DATASTORE->list_fetch( $list_id, $idx ) );
 } 
 
-sub hash_has_key {
-    my( $hash_id, $key ) = @_;
-    return $DATASTORE->hash_has_key( $hash_id, $key );
+sub list_insert {
+    my( $list_id, $val, $idx ) = @_;
+    return $DATASTORE->list_insert( $list_id, xform_in( $val ), $idx );
 }
 
+sub lock {
+    my( $id, $ref ) = @_;
+    my $oref = $LOCKER ? $LOCKER->lock_object( $id, $ref ) : $ref;
+    unless( $oref ) {
+	$oref = fetch( $id );
+	$ref->{DATA} = $oref->{DATA};
+    }
+    return $ref;
+} #lock
+
 sub package_methods {
     my $pkg = shift;
     my $methods = $Yote::ObjProvider::PKG_TO_METHODS{$pkg};
@@ -352,9 +353,15 @@ sub recycle_objects {
     return $DATASTORE->recycle_objects( @_ );
 } #recycle_objects
 
+sub remove_from {
+    my( $list_id, $item ) = @_;
+    return $DATASTORE->list_delete( $list_id, xform_in( $item ) );
+} #remove_from
+
 sub start_transaction {
     return $DATASTORE->start_transaction();
 }
+
 sub stow {
     my( $obj ) = @_;
 
@@ -409,7 +416,6 @@ sub stow {
 
 sub stow_all {
     my @odata;
-#    print STDERR "[$$ ".time()."] STOW ALL with " . scalar( keys %{$Yote::ObjProvider::DIRTY} ) . " dirty items.\n";
     for my $obj (values %{$Yote::ObjProvider::DIRTY} ) {
 	my $cls;
 	my $ref = ref( $obj );
@@ -426,6 +432,14 @@ sub stow_all {
     $Yote::ObjProvider::DIRTY = {};
 } #stow_all
 
+sub unlock {
+    my $id = shift;
+    if( $LOCKER ) {
+	return $LOCKER->unlock_objects( $id );
+    }
+    return;
+} #unlock
+
 sub xform_in {
     my $val = shift;
     if( ref( $val ) ) {
@@ -443,10 +457,6 @@ sub xform_out {
     return fetch( $val );
 }
 
-sub count {
-    my $container_id = shift;
-    return $DATASTORE->count( $container_id );
-}
 
 
 # ------------------------------------------------------------------------------------------
@@ -530,6 +540,11 @@ It is the only module to directly interact with the datastore layer.
 
 =item init - takes a hash of args, passing them to a new Yote::SQLite object and starting it up.
 
+=item attach_server( datalocker )
+
+This links a datalocker to this objprovider. This is called automatically by Yote::WebAppServer 
+which also serves as the locker. The datalocker is responsible for locking and unlocking objects.
+
 =back
 
 =head1 CLASS METHODS
@@ -538,28 +553,30 @@ It is the only module to directly interact with the datastore layer.
 
 =item commit_transaction( )
 
-=item count( container_id )
-
-returns the number of items in the given container
-
-
 Requests the data store used commit the transaction.
 
+=item count( container_id, args )
+
+Returns the number of items in the given container. Args are optional and are
+
+=over 4
+
+* search_fields - a list of fields to search for in collections of yote objects
+* search_terms - a list of terms to search for
+
+=back
+
 =item dirty( obj )
 
-Marks the object as dirty
+Marks the item as needing a save.
 
 =item disconnect( )
 
 Requests the data store used disconnect.
 
-=item encrypt_pass( pass_string )
+=item encrypt_pass( pass_string, handle_string )
 
-Returns a string of the argument encrypted.
-
-=item flush( id )
-
-Removes any object with the given ID from any cache
+Returns a string of the argument encrypted. This is 
 
 =item fetch( id )
 
@@ -567,11 +584,16 @@ Returns the array ref, hash ref or yote object specified by the numeric id or ha
 
 =item first_id( id )
 
-Returns the id of the first object in the system, the YoteRoot.
+Returns the id of the first object in the system, the YoteRoot. This may or may not be 
+numeric.
+
+=item flush( id )
+
+Removes any object with the given ID from any cache.
 
 =item flush_all_volatile()
 
-Clears out DIRTY and WEAK_REFS caches.
+Clears out all caches.
 
 =item get_id( obj )
 
@@ -580,26 +602,36 @@ if none had been assigned to it.
 
 =item hash_delete( hash_id, key )
 
-Removes the key from the hash given by the id
+Removes the key from the hash given by the id direclty from the database.
 
 =item hash_fetch( hash_id, key )
 
+Uses a database lookup to return the value for the key for the hash specified by hash_id.
+
 =item hash_has_key( hash_id, key )
 
+Uses a database lookup to return if the key for the hash specified by hash_id has a value.
+
 =item hash_insert( hash_id, key, value )
 
+Insert a key value pair directly into the database for the given hash_id.
+
 =item list_delete( list_id, idx )
 
+Uses the database to directly delete a given list element. This will cause the list to be reindexed.
+
 =item list_fetch( list_id, idx )
 
+Directly looks in the database to return the list element at the given index.
+
 =item list_insert( list_id, val, idx )
 
 Inserts the item into the list with an optional index. If not given, this inserts to the end of the list.
+This method will cause the list to be reindexed.
 
-=item make_server( datalocker )
-
-This is just used when multiple processes will be activatig the same data share.
+=item lock( id, ref )
 
+Requests that the object locker lock the object given by id and reference to this thread. Calling this blocks until the item is unlocked.
 
 =item package_methods( package_name )
 
@@ -612,7 +644,6 @@ Returns a paginated list or hash that is attached to the object specified by obj
 
 =over 4
 
-* name - name of data structure attached to this object.
 * search_fields - a list of fields to search for in collections of yote objects
 * search_terms - a list of terms to search for
 * sort_fields - a list of fields to sort by for collections of yote objects
@@ -624,15 +655,6 @@ Returns a paginated list or hash that is attached to the object specified by obj
 
 =back
 
-
-=item paginate_hash( hash_id, length, start )
-
-Returns a paginated hash reference
-
-=item paginate_list( list_id, length, start )
-
-Returns a paginated list reference
-
 =item power_clone( item )
 
 Returns a deep clone of the object. This will clone any object that is part of the yote system except for the yote root or any app (a Yote::AppRoot object)
@@ -641,9 +663,9 @@ Returns a deep clone of the object. This will clone any object that is part of t
 
 Recycles all objects in the range given if they cannot trace back a path to root.
 
-=item search_list
+=item remove_from( list_id, item )
 
-Returns a paginated search list
+Removes the items ( by value ) from the list with the given id.
 
 =item start_transaction( )
 
@@ -657,6 +679,10 @@ This saves the hash ref, array ref or yote object argument in the data store.
 
 Stows all objects that are marked as dirty. This is called automatically by the application server and need not be explicitly called.
 
+=item unlock( id )
+
+Requests the object locker release the object referenced by the given id that was locked to this thread.
+
 =item xform_in( value )
 
 Returns the internal yote storage for the value, be it a string/number value, or yote reference.
@@ -670,6 +696,8 @@ Returns the external value given the internal identifier. The external value can
 =head1 AUTHOR
 
 Eric Wolf
+coyocanid@gmail.com
+http://madyote.com
 
 =head1 LICENSE AND COPYRIGHT
 
