@@ -42,42 +42,28 @@ sub new {
     return bless {}, $class;
 }
 
-#
-# Sets up Initial database server and tables.
-#
-sub init_server {
-    my( $self, @args ) = @_;
-   Yote::ObjProvider::init_datastore( @args );
-} #init_server
-
 # ------------------------------------------------------------------------------------------
 #      * PUBLIC METHODS *
 # ------------------------------------------------------------------------------------------
 
 
-sub do404 {
-    my $self = shift;
-    print "HTTP/1.0 404 NOT FOUND\015\012Content-Type: text/html\n\nERROR : 404\n";
-}
-
-
-sub iolog {
+sub accesslog {
     my( $msg ) = @_;
     my $t = strftime "%Y-%m-%d %H:%M:%S", gmtime;
-#    print STDERR "[$$ ".time()."]$msg\n";
-    print $Yote::WebAppServer::IO "$t : $msg\n";
+    print $Yote::WebAppServer::ACCESS "$t : $msg\n";
 }
 
+
 sub errlog {
     my( $msg ) = @_;
     my $t = strftime "%Y-%m-%d %H:%M:%S", gmtime;
     print $Yote::WebAppServer::ERR "$t : $msg\n";
 }
 
-sub accesslog {
+sub iolog {
     my( $msg ) = @_;
     my $t = strftime "%Y-%m-%d %H:%M:%S", gmtime;
-    print $Yote::WebAppServer::ACCESS "$t : $msg\n";
+    print $Yote::WebAppServer::IO "$t : $msg\n";
 }
 
 sub lock_object {
@@ -87,7 +73,6 @@ sub lock_object {
 	return $ref;
     }
 
-#    print STDERR "[$$ ".time()."] LOCK REQ $obj_id \n";
     while( 1 ) {
 	my( @locked );
 	{
@@ -96,7 +81,6 @@ sub lock_object {
 
 	    if( ! $lockdata ) {
 		$oid2lockdata{ $obj_id } = "$$|";
-#		print STDERR "[$$ ".time()."] LOCKED $obj_id \n";
 		$self->{ LOCKED }{ $obj_id } = 1;
 		return $ref;
 	    }
@@ -104,19 +88,16 @@ sub lock_object {
 
 	    my( %waiters ) = map { $_ => 1 } @pids_waiting_for_this_object;
 
-#	    print STDERR "[$$ ".time()."] LOCKDATA : $lockdata\n";
-
 	    if( ! $locking_pid ) {
 		delete $waiters{$$}; #remove this from any waiting list
 		$oid2lockdata{ $obj_id } = join( '|', $$, $dirty_time, keys %waiters );
-#		print STDERR "[$$ ".time()."] LOCKED $obj_id \n";
 		$self->{ LOCKED }{ $obj_id } = 1;
 		if( $dirty_time &&  $Yote::ObjProvider::LAST_LOAD_TIME->{ $obj_id } && $Yote::ObjProvider::LAST_LOAD_TIME->{ $obj_id } <= $dirty_time ) {
-#		    print STDERR "[$$ ".time()."] RETURNING as dity time is now $obj_id \n";
+		    # means the object is dirty and should be reloaded
 		    return;
 		}
 		return $ref;
-	    }
+	    } # not locking pid
 	    
 	    # lock data indicates this obj is locked by an other pid. 
 	    # Check to make sure that pid isn't waiting on what we have locked, for that is deadlock
@@ -128,7 +109,6 @@ sub lock_object {
 		    if( Yote::ObjProvider::__is_dirty( $locked_oid ) ) {
 			$self->unlock_objects( keys %{ $self->{LOCKED} } );
 			Yote::ObjProvider::flush( map { Yote::ObjProvider::__is_dirty( $_ ) } keys %{ $self->{LOCKED} } );
-#			print STDERR Data::Dumper->Dump(["[$$ ".time()."] DEADLOCK"]);
 			die "__DEADLOCK__";
 		    }
 		}
@@ -146,11 +126,8 @@ sub lock_object {
 		$oid2lockdata{ $obj_id } = join( '|', $locking_pid, $dirty_time, $$, @pids_waiting_for_this_object );
 	    }
 	    
-#	    print STDERR "[$$ ".time()."] UPDATED oid2lockeddata for $obj_id : $oid2lockdata{ $obj_id }\n";
-	    
 	    unless( @locked ) {
 		lock( %oid2lockdata );
-#		print STDERR "[$$ ".time()."] NO LOCKED. WAITING ON $obj_id\n";
 		cond_wait( %oid2lockdata );
 	    }
 	} # scope for locked var
@@ -158,15 +135,144 @@ sub lock_object {
 	if( @locked ) {
 	    $self->unlock_objects( @locked );
 	    lock( %oid2lockdata );
-#	    print STDERR "[$$ ".time()."] JUST UNLOCKED. WAITING on $obj_id\n";
 	    cond_wait( %oid2lockdata );
 	}
     } #while loop
 } #lock_object
 
+
+sub shutdown {
+    my $self = shift;
+    accesslog( "Shutting down yote server" );
+    Yote::ObjProvider::start_transaction();
+    Yote::ObjProvider::stow_all();
+    Yote::ObjProvider::commit_transaction();
+    accesslog(  "Killing threads" );
+    $self->__stop_threads();
+    accesslog( "Shut down server thread" );
+} #shutdown
+
+sub start_server {
+    my( $self, @args ) = @_;
+    my $args = scalar(@args) == 1 ? $args[0] : { @args };
+
+    $self->{ args } = $args;
+    $self->{ args }{ webroot } ||= $self->{ args }{ yote_root } . '/html';
+    $self->{ args }{ upload }  ||= $self->{ args }{ webroot }   . '/upload';
+    $self->{ args }{ log_dir } ||= $self->{ args }{ yote_root } . '/log';
+    $self->{ args }{ port }    ||= 80;
+    $self->{ args }{ threads } ||= 10;
+
+    # make sure the filehelper knows where the data directory is
+    $Yote::WebAppServer::LOG_DIR       = $self->{args}{log_dir};
+    $Yote::WebAppServer::FILE_DIR      = $self->{args}{data_dir} . '/holding';
+    $Yote::WebAppServer::WEB_DIR       = $self->{args}{webroot};
+    $Yote::WebAppServer::UPLOAD_DIR    = $self->{args}{webroot}. '/uploads';
+    mkdir( $Yote::WebAppServer::FILE_DIR );
+    mkdir( $Yote::WebAppServer::WEB_DIR );
+    mkdir( $Yote::WebAppServer::UPLOAD_DIR );
+    mkdir( $Yote::WebAppServer::LOG_DIR );
+
+    open( $Yote::WebAppServer::IO,      '>>', "$Yote::WebAppServer::LOG_DIR/io.log" )
+		      && $Yote::WebAppServer::IO->autoflush;
+    open( $Yote::WebAppServer::ACCESS,  '>>', "$Yote::WebAppServer::LOG_DIR/access.log" )
+		      && $Yote::WebAppServer::ACCESS->autoflush;
+    open( $Yote::WebAppServer::ERR,     '>>', "$Yote::WebAppServer::LOG_DIR/error.log" )
+		      && $Yote::WebAppServer::ERR->autoflush;
+
+    Yote::ObjProvider::init( %$args );
+
+    # fork out for three starting threads
+    #   - one a multi forking server (parent class)
+    #   - one for a cron daemon inside of Yote. (PENDING)
+    #   - and the parent thread an event loop.
+
+    my $root = Yote::YoteRoot::fetch_root();
+
+    # check for default account and set its password from the config.
+    $root->_check_root( $args->{ root_account }, $args->{ root_password } );
+
+
+    # make sure the filehelper knows where the data directory is
+
+    # update @INC library list
+    my $paths = $root->get__application_lib_directories([]);
+    push @INC, Yote::ConfigData->config( 'yote_root' ), @$paths;
+
+    until( $self->{lsn} ) {
+	$self->{lsn} = new IO::Socket::INET(Listen => 10, LocalPort => $self->{args}{port});
+	unless( $self->{lsn} ) {
+	    if( $! =~ /Address already in use/i ) {
+		print STDERR "Address already in use. Retrying.\n";
+		sleep( 5 );
+	    } else {
+		die $!;
+	    }
+	}
+    }
+
+    print STDERR "Connected\n";
+    
+    $self->{ threads } = {};
+    
+    Yote::ObjProvider::attach_server( $self );
+    for( 1 .. $self->{args}{threads} ) {
+	$self->__start_server_thread;
+    } #creating threads
+
+    my $cron = $root->get__crond();
+    Yote::ObjProvider::flush_all_volatile();
+    $self->__unlock_all();
+    
+
+    while( 1 ) {
+	sleep( 5 );
+	my $threads = $self->{ threads };
+	for my $thread ( values %$threads ) {
+	    if( $thread->is_joinable() ) {
+		delete $threads->{ $thread->tid() };
+		$thread->join();
+	    }
+	}
+	while( scalar( keys %$threads ) < $self->{ args }{ threads } ) {
+	    $self->__start_server_thread;
+	}
+	
+	my $cron_entries = $cron->entries();
+	Yote::ObjProvider::flush_all_volatile();
+	$self->__unlock_all();
+
+	for my $entry (@$cron_entries) {
+	    threads->new( sub {
+		$cron->_mark_done( $entry );
+		print STDERR "Starting cron thread " . threads->tid() . "\n";
+		my $script = $entry->get_script();
+		print STDERR "EVAL $script\n";
+		eval "$script";
+		print STDERR "Done EVAL\n";
+		if( $@ ) {
+		    print STDERR "Error in Cron : $@ $!\n";
+		} 
+		
+		$self->__check_locked_for_dirty();
+		Yote::ObjProvider::start_transaction();
+		Yote::ObjProvider::stow_all();
+		Yote::ObjProvider::flush_all_volatile();
+		Yote::ObjProvider::commit_transaction();
+		$self->__unlock_all();
+		print STDERR "Done cron thread " . threads->tid() . "\n";
+			  } ); #done with cron entry thread
+	} #each cron entry
+    } #endless loop
+
+    __stop_threads();
+
+   Yote::ObjProvider::disconnect();
+
+} #start_server
+
 sub unlock_objects {
     my( $self, @objs ) = @_;
-    return if $Yote::ObjProvider::LOCK_NEVER;
     @objs = grep { $self->{ LOCKED }{ $_ } } @objs;
     if( @objs ) {
 	lock( %oid2lockdata );
@@ -179,7 +285,11 @@ sub unlock_objects {
     }
 } #unlock_objects
 
-sub check_locked_for_dirty {
+# ------------------------------------------------------------------------------------------
+#      * PRIVATE METHODS *
+# ------------------------------------------------------------------------------------------
+
+sub __check_locked_for_dirty {
     my( $self  ) = @_;
     my( @dirty_oids ) = grep { Yote::ObjProvider::__is_dirty( $_ ) } keys %{ $self->{LOCKED} || {} };
     if( @dirty_oids ) {
@@ -190,25 +300,160 @@ sub check_locked_for_dirty {
 	    $oid2lockdata{ $dirty_oid } = join( '|', $locking_pid, $t, @pids_waiting_for_this_object );
 	}
     } #if dirty
+} #__check_locked_for_dirty
+
+sub _do404 {
+    my $self = shift;
+    print "HTTP/1.0 404 NOT FOUND\015\012Content-Type: text/html\n\nERROR : 404\n";
 }
 
-sub unlock_all {
-    my( $self  ) = @_;
-    $self->unlock_objects( keys %{ $self->{LOCKED} || {} } );
-    $self->{LOCKED} = {};
-}
 
-#
-# Called when a request is made. This does an initial parsing and
-# sends a data structure to _process_command.
-#
-# Commands are sent with a single HTTP request parameter : m for message.
-#
-#
-# This ads a command to the list of commands. If
-#
-#sub process_request {
-sub process_http_request {
+
+sub __stop_threads {
+    my $self = shift;
+    $self->{watchdog_thread}->kill if $self->{watchdog_thread} && $self->{watchdog_thread}->is_running;
+    for my $thread (values %{$self->{threads}}) {
+	$thread->join if $thread && $thread->is_joinable;
+	$thread->kill if $thread && $thread->is_running;
+    }
+} #__stop_threads
+
+sub __start_server_thread {
+    my $self = shift;
+
+    my $new_thread = threads->new(
+	sub {
+	    print STDERR "Starting server thread " . threads->tid() . "\n";
+	    $SIG{PIPE} = sub { 
+		print STDERR "Thread $$ got sig pipe. Exiting\n";
+		$self->{lsn}->close() if $self->{lsn};
+		threads->exit()
+	    };
+	    unless( $self->{lsn} ) {
+		threads->exit();
+	    }
+	    
+	    open( $Yote::WebAppServer::IO,      '>>', "$Yote::WebAppServer::LOG_DIR/io.log" ) 
+		&& $Yote::WebAppServer::IO->autoflush;
+	    open( $Yote::WebAppServer::ACCESS,  '>>', "$Yote::WebAppServer::LOG_DIR/access.log" )
+		&& $Yote::WebAppServer::ACCESS->autoflush;
+	    open( $Yote::WebAppServer::ERR,     '>>', "$Yote::WebAppServer::LOG_DIR/error.log" )
+		&& $Yote::WebAppServer::ERR->autoflush;
+	    
+	    while( my $fh = $self->{lsn}->accept ) {
+		eval {
+		    $ENV{ REMOTE_ADDR } = $fh->peerhost;
+		    $self->__process_http_request( $fh );
+		};
+		if( $@ ) {
+		    $self->{lsn}->close() if $self->{lsn};
+		    last;
+		}
+		$fh->close();
+	    } #main loop
+	} ); #new thread
+    $self->{ threads }{ $new_thread->tid() } = $new_thread;
+    
+} #__start_server_thread
+
+sub __process_command {
+    my( $self, $command ) = @_;
+    my $resp;
+
+    eval {
+        my $obj_id = $command->{oi};
+        my $app_id = $command->{ai};
+
+        my $app         = Yote::ObjProvider::fetch( $app_id ) || Yote::YoteRoot::fetch_root();
+
+        my $data        = __translate_data( from_json( MIME::Base64::decode( $command->{d} ) )->{d} );
+
+#	iolog( "  * CMD IN $$ : " . Data::Dumper->Dump( [ $command ] ) );
+
+#	iolog( "  * DATA IN $$ : " . Data::Dumper->Dump( [ $data ] ) );
+
+        my $login       = $app->token_login( $command->{t}, undef, $command->{e} );
+	my $guest_token = $command->{gt};
+	$command->{e}{GUEST_TOKEN} = $guest_token;
+
+	# security check
+	unless( Yote::ObjManager::allows_access( $obj_id, $app, $login, $guest_token ) ) {
+	    accesslog( "INVALID ACCCESS ATTEMPT for $obj_id from $command->{e}{ REMOTE_ADDR }" );
+	    die "Access Error";
+	}
+
+        my $app_object = Yote::ObjProvider::fetch( $obj_id ) || $app;
+        my $action     = $command->{a};
+
+	die "Access Error" if $action =~ /^([gs]et|add_(once_)?to_|remove_(all_)?from)_/; # set may not be called directly on an object.
+        my $account;
+        if( $login ) {
+            $account = $app->__get_account( $login );
+	    $account->set_login( $login ); # security measure to make sure login can't be overridden by a subclass of account
+	    $login->add_once_to__accounts( $account );
+        }
+
+        my $ret = $app_object->$action( $data, $account, $command->{e} );
+
+	my $dirty_delta = Yote::ObjManager::fetch_dirty( $login, $guest_token );
+
+	my( $dirty_data );
+	if( @$dirty_delta ) {
+	    $dirty_data = {};
+	    for my $d_id ( @$dirty_delta ) {
+		my $dobj = Yote::ObjProvider::fetch( $d_id );
+		if( ref( $dobj ) eq 'ARRAY' ) {
+		    $dirty_data->{$d_id} = { map { $_ => Yote::ObjProvider::xform_in( $dobj->[$_] ) } (0..$#$dobj) };
+		} elsif( ref( $dobj ) eq 'HASH' ) {
+		    $dirty_data->{$d_id} = { map { $_ => Yote::ObjProvider::xform_in( $dobj->{ $_ } ) } keys %$dobj };
+		} else {
+		    $dirty_data->{$d_id} = { map { $_ => $dobj->{DATA}{$_} } grep { $_ !~ /^_/ } keys %{$dobj->{DATA}} };
+		}
+		for my $val (values %{ $dirty_data->{$d_id} } ) {
+		    if( index( $val, 'v' ) != 0 ) {
+			Yote::ObjManager::register_object( $val, $login ? $login->{ID} : $guest_token );
+		    }
+		}
+	    }
+	} #if there was a dirty delta
+        $resp = $dirty_data ? { r => __obj_to_response( $ret, $login, $guest_token ), d => $dirty_data } : { r => __obj_to_response( $ret, $login, $guest_token ) };
+    };
+    if( $@ ) {
+	my $err = $@;
+	print STDERR Data::Dumper->Dump(["ERRRR $@"]);
+	if( $err =~ /^__DEADLOCK__/ ) {
+	    iolog( "DEADLOCK TO RETRY $$ : $@" );
+	    # if a deadlock condition was detected. back out of any changes and retry
+	    # now this could become an issue if things deadlock really really often as the stack would fill up.
+#	    print STDERR "[$$ ".time()."] DEADLOCK DETECTED\n";
+	    $self->__unlock_all();
+	    return $self->__process_command( $command );
+	}
+	$err =~ s/at \/\S+\.pm.*//s;
+        errlog( "ERROR : $@" );
+	iolog( "ERROR : $@" );
+        $resp = { err => $err, r => '' };
+    }
+    
+    $resp = to_json( $resp );
+
+    $self->__check_locked_for_dirty();
+    Yote::ObjProvider::start_transaction();
+    Yote::ObjProvider::stow_all();
+    Yote::ObjProvider::flush_all_volatile();
+    Yote::ObjProvider::commit_transaction();
+    $self->__unlock_all();
+
+    ### SEND BACK $resp
+    iolog( " * DATA BACK $$ : $resp" );
+    #
+    # Send return value back to the caller if its waiting for it.
+    #
+    return $resp
+
+} #__process_command
+
+sub __process_http_request {
     my( $self, $soc ) = @_;
 
     my $req = <$soc>;
@@ -222,7 +467,7 @@ sub process_http_request {
 
     my $content_length = $ENV{CONTENT_LENGTH};
     if( $content_length > 5_000_000 ) { #make this into a configurable field
-	$self->do404();
+	$self->_do404();
 	close( $soc );
 	return;
     }
@@ -272,7 +517,7 @@ sub process_http_request {
 	    $app_id ||= Yote::ObjProvider::first_id();
 	}
 	else {
-	    my $vars = Yote::FileHelper::__ingest( _parse_form( $soc ) );
+	    my $vars = Yote::FileHelper::__ingest( __parse_form( $soc ) );
 	    $data        = $vars->{d};
 	    $token       = $vars->{t};
 	    $guest_token = $vars->{gt};
@@ -282,7 +527,7 @@ sub process_http_request {
 	    $app_id      = pop( @path ) || Yote::ObjProvider::first_id();
 	}
 
-        my $result = $self->_process_command( {
+        my $result = $self->__process_command( {
             a  => $action,
             ai => $app_id,
             d  => $data,
@@ -355,297 +600,18 @@ sub process_http_request {
 	    #accesslog( "200 : $dest");
 	} else {
 	    accesslog( "404 NOT FOUND (".threads->tid().") : $@,$! [$root/$dest]");
-	    $self->do404();
+	    $self->_do404();
 	}
 	close( $soc );
 	return;
     } #serve html
 
-} #process_request
-
-
-sub shutdown {
-    my $self = shift;
-    accesslog( "Shutting down yote server" );
-    Yote::ObjProvider::start_transaction();
-    Yote::ObjProvider::stow_all();
-    Yote::ObjProvider::commit_transaction();
-    accesslog(  "Killing threads" );
-    $self->_stop_threads();
-    accesslog( "Shut down server thread" );
-} #shutdown
-
-sub start_server {
-    my( $self, @args ) = @_;
-    my $args = scalar(@args) == 1 ? $args[0] : { @args };
-
-    $self->{ args } = $args;
-    $self->{ args }{ webroot } ||= $self->{ args }{ yote_root } . '/html';
-    $self->{ args }{ upload }  ||= $self->{ args }{ webroot }   . '/upload';
-    $self->{ args }{ log_dir } ||= $self->{ args }{ yote_root } . '/log';
-    $self->{ args }{ port }    ||= 80;
-    $self->{ args }{ threads } ||= 10;
-
-    # make sure the filehelper knows where the data directory is
-    $Yote::WebAppServer::LOG_DIR       = $self->{args}{log_dir};
-    $Yote::WebAppServer::FILE_DIR      = $self->{args}{data_dir} . '/holding';
-    $Yote::WebAppServer::WEB_DIR       = $self->{args}{webroot};
-    $Yote::WebAppServer::UPLOAD_DIR    = $self->{args}{webroot}. '/uploads';
-    mkdir( $Yote::WebAppServer::FILE_DIR );
-    mkdir( $Yote::WebAppServer::WEB_DIR );
-    mkdir( $Yote::WebAppServer::UPLOAD_DIR );
-    mkdir( $Yote::WebAppServer::LOG_DIR );
-
-    open( $Yote::WebAppServer::IO,      '>>', "$Yote::WebAppServer::LOG_DIR/io.log" )
-		      && $Yote::WebAppServer::IO->autoflush;
-    open( $Yote::WebAppServer::ACCESS,  '>>', "$Yote::WebAppServer::LOG_DIR/access.log" )
-		      && $Yote::WebAppServer::ACCESS->autoflush;
-    open( $Yote::WebAppServer::ERR,     '>>', "$Yote::WebAppServer::LOG_DIR/error.log" )
-		      && $Yote::WebAppServer::ERR->autoflush;
-
-    Yote::ObjProvider::init( %$args );
-
-    # fork out for three starting threads
-    #   - one a multi forking server (parent class)
-    #   - one for a cron daemon inside of Yote. (PENDING)
-    #   - and the parent thread an event loop.
-
-    my $root = Yote::YoteRoot::fetch_root();
-
-    # check for default account and set its password from the config.
-    $root->_check_root( $args->{ root_account }, $args->{ root_password } );
-
-
-    # make sure the filehelper knows where the data directory is
-
-    # update @INC library list
-    my $paths = $root->get__application_lib_directories([]);
-    push @INC, Yote::ConfigData->config( 'yote_root' ), @$paths;
-
-    until( $self->{lsn} ) {
-	$self->{lsn} = new IO::Socket::INET(Listen => 10, LocalPort => $self->{args}{port});
-	unless( $self->{lsn} ) {
-	    if( $! =~ /Address already in use/i ) {
-		print STDERR "Address already in use. Retrying.\n";
-		sleep( 5 );
-	    } else {
-		die $!;
-	    }
-	}
-    }
-
-    print STDERR "Connected\n";
-    
-    $self->{ threads } = {};
-    
-    Yote::ObjProvider::make_server( $self );
-    for( 1 .. $self->{args}{threads} ) {
-	$self->_start_server_thread;
-    } #creating threads
-
-    my $cron = $root->get__crond();
-    Yote::ObjProvider::flush_all_volatile();
-    $self->unlock_all();
-    
-
-    while( 1 ) {
-	sleep( 5 );
-	my $threads = $self->{ threads };
-	for my $thread ( values %$threads ) {
-	    if( $thread->is_joinable() ) {
-		delete $threads->{ $thread->tid() };
-		$thread->join();
-	    }
-	}
-	while( scalar( keys %$threads ) < $self->{ args }{ threads } ) {
-	    $self->_start_server_thread;
-	}
-	
-	my @cron_entries = $cron->entries();
-	Yote::ObjProvider::flush_all_volatile();
-	$self->unlock_all();
-
-	for my $entry (@cron_entries) {
-	    threads->new( sub {
-		$cron->mark_done( $entry );
-		print STDERR "Starting cron thread " . threads->tid() . "\n";
-		my $script = $entry->get_script();
-		print STDERR "EVAL $script\n";
-		eval "$script";
-		print STDERR "Done EVAL\n";
-		if( $@ ) {
-		    print STDERR "Error in Cron : $@ $!\n";
-		} 
-		
-		$self->check_locked_for_dirty();
-		Yote::ObjProvider::start_transaction();
-		Yote::ObjProvider::stow_all();
-		Yote::ObjProvider::flush_all_volatile();
-		Yote::ObjProvider::commit_transaction();
-		$self->unlock_all();
-		print STDERR "Done cron thread " . threads->tid() . "\n";
-			  } ); #done with cron entry thread
-	} #each cron entry
-    } #endless loop
-
-    _stop_threads();
-
-   Yote::ObjProvider::disconnect();
-
-} #start_server
-
-# ------------------------------------------------------------------------------------------
-#      * PRIVATE METHODS *
-# ------------------------------------------------------------------------------------------
-
-sub _stop_threads {
-    my $self = shift;
-    $self->{watchdog_thread}->kill if $self->{watchdog_thread} && $self->{watchdog_thread}->is_running;
-    for my $thread (values %{$self->{threads}}) {
-	$thread->join if $thread && $thread->is_joinable;
-	$thread->kill if $thread && $thread->is_running;
-    }
-}
-
-sub _start_server_thread {
-    my $self = shift;
-
-    my $new_thread = threads->new(
-	sub {
-	    print STDERR "Starting server thread " . threads->tid() . "\n";
-	    $SIG{PIPE} = sub { 
-		print STDERR "Thread $$ got sig pipe. Exiting\n";
-		$self->{lsn}->close() if $self->{lsn};
-		threads->exit()
-	    };
-	    unless( $self->{lsn} ) {
-		threads->exit();
-	    }
-	    
-	    open( $Yote::WebAppServer::IO,      '>>', "$Yote::WebAppServer::LOG_DIR/io.log" ) 
-		&& $Yote::WebAppServer::IO->autoflush;
-	    open( $Yote::WebAppServer::ACCESS,  '>>', "$Yote::WebAppServer::LOG_DIR/access.log" )
-		&& $Yote::WebAppServer::ACCESS->autoflush;
-	    open( $Yote::WebAppServer::ERR,     '>>', "$Yote::WebAppServer::LOG_DIR/error.log" )
-		&& $Yote::WebAppServer::ERR->autoflush;
-	    
-	    while( my $fh = $self->{lsn}->accept ) {
-		eval {
-		    $ENV{ REMOTE_ADDR } = $fh->peerhost;
-		    $self->process_http_request( $fh );
-		};
-		if( $@ ) {
-		    $self->{lsn}->close() if $self->{lsn};
-		    last;
-		}
-		$fh->close();
-	    } #main loop
-	} ); #new thread
-    $self->{ threads }{ $new_thread->tid() } = $new_thread;
-    
-} #_start_server_thread
-
-sub _process_command {
-    my( $self, $command ) = @_;
-    my $resp;
-
-    eval {
-        my $obj_id = $command->{oi};
-        my $app_id = $command->{ai};
-
-        my $app         = Yote::ObjProvider::fetch( $app_id ) || Yote::YoteRoot::fetch_root();
-
-        my $data        = _translate_data( from_json( MIME::Base64::decode( $command->{d} ) )->{d} );
-
-#	iolog( "  * CMD IN $$ : " . Data::Dumper->Dump( [ $command ] ) );
-
-#	iolog( "  * DATA IN $$ : " . Data::Dumper->Dump( [ $data ] ) );
-
-        my $login       = $app->token_login( $command->{t}, undef, $command->{e} );
-	my $guest_token = $command->{gt};
-	$command->{e}{GUEST_TOKEN} = $guest_token;
-
-	# security check
-	unless( Yote::ObjManager::allows_access( $obj_id, $app, $login, $guest_token ) ) {
-	    accesslog( "INVALID ACCCESS ATTEMPT for $obj_id from $command->{e}{ REMOTE_ADDR }" );
-	    die "Access Error";
-	}
-
-        my $app_object = Yote::ObjProvider::fetch( $obj_id ) || $app;
-        my $action     = $command->{a};
-
-	die "Access Error" if $action =~ /^([gs]et|add_(once_)?to_|remove_(all_)?from)_/; # set may not be called directly on an object.
-        my $account;
-        if( $login ) {
-            $account = $app->__get_account( $login );
-	    $account->set_login( $login ); # security measure to make sure login can't be overridden by a subclass of account
-	    $login->add_once_to__accounts( $account );
-        }
-
-        my $ret = $app_object->$action( $data, $account, $command->{e} );
-
-	my $dirty_delta = Yote::ObjManager::fetch_dirty( $login, $guest_token );
-
-	my( $dirty_data );
-	if( @$dirty_delta ) {
-	    $dirty_data = {};
-	    for my $d_id ( @$dirty_delta ) {
-		my $dobj = Yote::ObjProvider::fetch( $d_id );
-		if( ref( $dobj ) eq 'ARRAY' ) {
-		    $dirty_data->{$d_id} = { map { $_ => Yote::ObjProvider::xform_in( $dobj->[$_] ) } (0..$#$dobj) };
-		} elsif( ref( $dobj ) eq 'HASH' ) {
-		    $dirty_data->{$d_id} = { map { $_ => Yote::ObjProvider::xform_in( $dobj->{ $_ } ) } keys %$dobj };
-		} else {
-		    $dirty_data->{$d_id} = { map { $_ => $dobj->{DATA}{$_} } grep { $_ !~ /^_/ } keys %{$dobj->{DATA}} };
-		}
-		for my $val (values %{ $dirty_data->{$d_id} } ) {
-		    if( index( $val, 'v' ) != 0 ) {
-			Yote::ObjManager::register_object( $val, $login ? $login->{ID} : $guest_token );
-		    }
-		}
-	    }
-	} #if there was a dirty delta
-        $resp = $dirty_data ? { r => __obj_to_response( $ret, $login, $guest_token ), d => $dirty_data } : { r => __obj_to_response( $ret, $login, $guest_token ) };
-    };
-    if( $@ ) {
-	my $err = $@;
-	print STDERR Data::Dumper->Dump(["ERRRR $@"]);
-	if( $err =~ /^__DEADLOCK__/ ) {
-	    iolog( "DEADLOCK TO RETRY $$ : $@" );
-	    # if a deadlock condition was detected. back out of any changes and retry
-	    # now this could become an issue if things deadlock really really often as the stack would fill up.
-#	    print STDERR "[$$ ".time()."] DEADLOCK DETECTED\n";
-	    $self->unlock_all();
-	    return $self->_process_command( $command );
-	}
-	$err =~ s/at \/\S+\.pm.*//s;
-        errlog( "ERROR : $@" );
-	iolog( "ERROR : $@" );
-        $resp = { err => $err, r => '' };
-    }
-    
-    $resp = to_json( $resp );
-
-    $self->check_locked_for_dirty();
-    Yote::ObjProvider::start_transaction();
-    Yote::ObjProvider::stow_all();
-    Yote::ObjProvider::flush_all_volatile();
-    Yote::ObjProvider::commit_transaction();
-    $self->unlock_all();
-
-    ### SEND BACK $resp
-    iolog( " * DATA BACK $$ : $resp" );
-    #
-    # Send return value back to the caller if its waiting for it.
-    #
-    return $resp
-
-} #_process_command
+} #__process_http_request
 
 #
 # 
 #
-sub _parse_form {
+sub __parse_form {
     my $soc = shift;
     my $content_length = $ENV{CONTENT_LENGTH} || $ENV{'HTTP_CONTENT-LENGTH'} || $ENV{HTTP_CONTENT_LENGTH};
     my( $finding_headers, $finding_content, %content_data, %post_data, %file_helpers, $fn, $content_type );
@@ -713,14 +679,14 @@ sub _parse_form {
 #
 # Translates from vValue and reference_id to values and references
 #
-sub _translate_data {
+sub __translate_data {
     my( $val ) = @_;
 
     if( ref( $val ) eq 'HASH' ) { #from javacript object, or hash. no fields starting with underscores accepted
-        return { map {  $_ => _translate_data( $val->{$_} ) } grep { index( $_, '_' ) != 0 } keys %$val };
+        return { map {  $_ => __translate_data( $val->{$_} ) } grep { index( $_, '_' ) != 0 } keys %$val };
     }
     elsif( ref( $val ) eq 'ARRAY' ) { #from javacript object, or hash. no fields starting with underscores accepted
-        return [ map {  _translate_data( $_ ) } @$val ];
+        return [ map {  __translate_data( $_ ) } @$val ];
     }
     return unless $val;
     if( index($val,'v') == 0 ) {
@@ -737,7 +703,7 @@ sub _translate_data {
     else {
 	return Yote::ObjProvider::fetch( $val );
     }
-} #_translate_data
+} #__translate_data
 
 #
 # Converts scalar, yote object, hash or array to data for returning.
@@ -829,13 +795,20 @@ sub __transform_data_no_id {
 } #__transform_data_no_id
 
 
+sub __unlock_all {
+    my( $self  ) = @_;
+    $self->unlock_objects( keys %{ $self->{LOCKED} || {} } );
+    $self->{LOCKED} = {};
+}
+
+
 1;
 
 __END__
 
 =head1 NAME
 
-Yote::WebAppServer - is a library used for creating prototype applications for the web.
+Yote::WebAppServer - This is the app server engine that provides server threads and all javascript perl IO.
 
 =head1 SYNOPSIS
 
@@ -850,8 +823,6 @@ $server->start_server();
 This starts an appslication server running on a specified port and hooked up to a specified datastore.
 Additional parameters are passed to the datastore.
 
-The server set up uses Net::Server::Fork receiving and sending messages on multiple threads. These threads queue up the messages for a single threaded event loop to make things thread safe. Incomming requests can either wait for their message to be processed or return immediately.
-
 =head1 PUBLIC METHODS
 
 =over 4
@@ -860,14 +831,6 @@ The server set up uses Net::Server::Fork receiving and sending messages on multi
 
 Write the message to the access log
 
-=item check_locked_for_dirty()
-
-Checks items that are dirty and notes that in the inter process communications.
-
-=item do404
-
-Return a 404 not found page and exit.
-
 =item errlog( msg )
 
 Write the message to the error log
@@ -876,41 +839,33 @@ Write the message to the error log
 
 Writes to an IO log for client server communications
 
-=item init_server
-
 =item lock_object( obj_id )
 
 Locks the given object id for use by this process only until it is unlocked.
 
-=item unlock_all()
-
-Unlocks all objects locked by this process and notifies other processes
-
-=item unlock_objects( objlist )
-
-Unlocked items in the given list and notifices other processes.
-
 =item new
 
 Returns a new WebAppServer.
 
-Sets up Initial database server and tables.
-
-=item process_http_request( )
-
-This implements Net::Server::HTTP and is called automatically for each incomming request.
-
 =item shutdown( )
 
 Shuts down the yote server, saving all unsaved items.
 
 =item start_server( )
 
+Starts the yote server and launches server threads and activates cron.
+
+=item unlock_objects( @list_of_obj_ids )
+
+Unlocks the objects referenced by the ids passed in. 
+
 =back
 
 =head1 AUTHOR
 
 Eric Wolf
+coyocanid@gmail.com
+http://madyote.com
 
 =head1 LICENSE AND COPYRIGHT
 
