@@ -10,6 +10,8 @@ no warnings 'uninitialized';
 
 use Yote::Cron;
 use Yote::Login;
+use Yote::RootObj;
+use Yote::UserObj;
 
 use Email::Valid;
 use Mail::Sender;
@@ -194,6 +196,40 @@ sub logout {
 	$login->set__token();
     }
 } #logout
+
+#
+# Transforms the login into a login with root privs. Do not use lightly.
+#
+sub make_root {
+    my( $self, $login, $acct ) = @_;
+    die "Access Error" unless $acct->is_root();
+    $login->set__is_root( 1 );
+    return;
+} #make_root
+
+sub new_obj {
+    my( $self, $data, $acct ) = @_;
+    my $ret = new Yote::Obj( ref( $data ) ? $data : undef );
+    $ret->set___creator( $acct );
+    return $ret;
+} #new_obj
+
+sub new_root_obj {
+    my( $self, $data, $acct ) = @_;
+    return "Access Error" unless $acct->get_login()->is_root();
+    my $ret = new Yote::RootObj( ref( $data ) ? $data : undef );
+    $ret->set___creator( $acct );
+    return $ret;
+} #new_root_obj
+
+sub new_user_obj {
+    my( $self, $data, $acct ) = @_;
+    my $ret = new Yote::UserObj( ref( $data ) ? $data : undef );
+    $ret->set___creator( $acct );
+    return $ret;
+
+} #new_user_obj
+
 #
 # Used to wipe and reset a whole app's data. Use with caution
 # and can only be used by the superuser.
@@ -209,9 +245,6 @@ sub purge_app {
     die "Permissions Error";
 } #purge_app
 
-
-
-
 #
 # Sends an email to the address containing a link to reset password.
 #
@@ -312,6 +345,18 @@ sub remove_login {
     
 } #remove_login
 
+#
+# Removes root privs from a login. Do not use lightly. Does not remove the last root if there is one
+#
+sub remove_root {
+    my( $self, $login, $acct ) = @_;
+    die "Access Error" unless $acct->is_root();
+    die "Cannot remove master root account" if $login->get__is_master_root();
+    $login->set__is_root( 0 );
+    return;
+} #remove_root
+
+
 # ------------------------------------------------------------------------------------------
 #      * PRIVATE METHODS *
 # ------------------------------------------------------------------------------------------
@@ -343,27 +388,6 @@ sub _check_root {
     return $root_login;
 } #_check_root
 
-#
-# Transforms the login into a login with root privs. Do not use lightly.
-#
-sub make_root {
-    my( $self, $login, $acct ) = @_;
-    die "Access Error" unless $acct->is_root();
-    $login->set__is_root( 1 );
-    return;
-} #make_root
-
-#
-# Removes root privs from a login. Do not use lightly. Does not remove the last root if there is one
-#
-sub remove_root {
-    my( $self, $login, $acct ) = @_;
-    die "Access Error" unless $acct->is_root();
-    die "Cannot remove master root account" if $login->get__is_master_root();
-    $login->set__is_root( 0 );
-    return;
-} #remove_root
-
 
 #
 # Create token and store with the account and return it.
@@ -385,14 +409,22 @@ Yote::YoteRoot
 
 =head1 DESCRIPTION
 
-The yote root is the main app of the class. It is also always object id 1 and sits at the head of the yote data tree. Yote::YoteRoot is a subclass of Yote::AppRoot.
-
-=head1 DATA 
+This is the first object and the root of the object graph. It stores user logins and stores the apps themselves.
 
 =head1 PUBLIC API METHODS
 
 =over 4
 
+=item create_login( args )
+
+Create a login with the given client supplied args : h => handle, e => email, p => password.
+This checks to make sure handle and email address are not already taken. 
+This is invoked by the javascript call $.yote.create_login( handle, password, email )
+
+=item cron
+
+Returns the cron. Only a root login may call this.
+
 =item fetch( id_list )
 
 Returns the list of the objects to the client provided the client is authroized to receive them.
@@ -409,10 +441,6 @@ Returns the singleton root object. It creates it if it has not been created.
 
 Creates and returns a guest token, associating it with the calling IP address.
 
-=item is_root
-
-Returns true if the account passed in is a root account.
-
 =item login( { h: handle, p : password } )
 
 Attempts to log the account in with the given credentials. Returns a data structre with 
@@ -426,6 +454,18 @@ Invalidates the tokens of the currently logged in user.
 
 Takes a login as an argument and makes it root. Throws access error if the callee is not root.
 
+=item new_obj( optional_data_hash )
+
+Returns a new yote object, initialized with the optional has reference.
+
+=item new_root_obj( optional_data_hash )
+
+Returns a new root yote object, initialized with the optional has reference.
+
+=item new_user_obj( optional_data_hash )
+
+Returns a new user yote object, initialized with the optional has reference.
+
 =item init - takes a hash of args, passing them to a new Yote::SQLite object and starting it up.
 
 =item purge_app
@@ -449,21 +489,55 @@ Purges the login account from the system if its credentials are verified. It mov
 
 Removes the root bit from the login.
 
-=item create_login( args )
+=back
 
-Create a login with the given client supplied args : h => handle, e => email, p => password.
-This checks to make sure handle and email address are not already taken. 
-This is invoked by the javascript call $.yote.create_login( handle, password, email )
+=head1 PRIVATE DATA FIELDS
 
-=item cron
+=over 4
 
-Returns the cron. Only a root login may call this.
+=item _apps
+
+Hash of classname to app singleton.
+
+=item _emails
+
+Hash of email to login object.    
+
+=item _handles
+
+Hash of handle to login object.
+
+=item _crond
+
+A singleton instance of the Cron.
+
+=item _application_lib_directories
+
+A list of directories that Yote will use to look for perl packages.
+
+=item __ALLOWS
+
+A hash of recipient ids to a hash of objects ids whos clients are allowed to access this object.
+
+=item __ALLOWS_REV
+
+A hash of object ids to a hash of recipient ibds whos clients are allowed to access this object.
+
+=item __DIRTY
+
+A hash of recipient ids to a hash of objects ids that need refreshing for that recipient.
+
+=item _account_roots
+
+This is a hash of login ID to account.
 
 =back
 
 =head1 AUTHOR
 
 Eric Wolf
+coyocanid@gmail.com
+http://madyote.com
 
 =head1 LICENSE AND COPYRIGHT
 
